import { Component, linkEvent } from 'inferno';
import * as moment from 'moment';

import { endpoint } from '../env';
import { SearchParams, Results } from '../interfaces';
import { convertCsvToJson, humanFileSize, magnetLink } from '../utils';

interface State {
  results: Results;
  searchParams: SearchParams;
  searching: Boolean;
}

export class Search extends Component<any, State> {

  state: State = {
    results: {
      torrents: []
    },
    searchParams: {
      q: "",
      page: 1
    },
    searching: false
  };

  constructor(props, context) {
    super(props, context);
  }

  componentDidMount() {
    this.state.searchParams = {
      page: Number(this.props.match.params.page),
      q: this.props.match.params.q
    }
    this.search();
  }

  // Re-do search if the props have changed
  componentDidUpdate(lastProps, lastState, snapshot) {
    if (lastProps.match && lastProps.match.params !== this.props.match.params) {
      this.state.searchParams = {
        page: Number(this.props.match.params.page),
        q: this.props.match.params.q
      }
      this.search();
    }

  }

  search() {
    if (!!this.state.searchParams.q) {
      this.setState({ searching: true, results: { torrents: [] } });
      this.fetchData(this.state.searchParams)
        .then(results => {
          if (!!results) {
            this.setState({
              results: results
            });
          }
        }).catch(error => {
          console.error('request failed', error);
        }).then(() => this.setState({ searching: false }));
    } else {
      this.setState({ results: { torrents: [] } });
    }
  }

  fetchData(searchParams: SearchParams): Promise<Results> {
    let q = encodeURI(searchParams.q);
    return fetch(`${endpoint}/service/search?q=${q}&page=${searchParams.page}&size=20`)
      .then(data => data.text())
      .then(csv => convertCsvToJson(csv));
  }

  render() {
    return (
      <div>{
          this.state.searching ?
            this.spinner()
            :
            this.state.results.torrents[0] ? this.table() : this.noResults()
        }</div>
    );
  }

  spinner() {
    return (
      <div class="text-center m-5 p-5">
        <i class="fas fa-spinner fa-spin fa-5x"></i>
      </div>
    );
  }

  noResults() {
    return (
      <div class="text-center m-5 p-5">
        <h1>No Results</h1>
      </div>
    )
  }

  table() {
    return (
      <div>
        <table class="table table-fixed table-hover table-sm table-striped table-hover-purple table-padding">
          <thead>
            <tr>
              <th class="border-0 align-middle search-name-col">Name</th>
              <th class="border-0 align-middle text-right">Size</th>
              <th class="border-0 align-middle text-right"><i class="fas fa-fw fa-arrow-up text-success d-inline d-md-none"></i><span class="d-none d-md-inline">Seeds</span></th>
              <th class="border-0 align-middle text-right d-none d-md-table-cell">Leeches</th>
              <th class="border-0 align-middle text-right d-none d-md-table-cell">Created</th>
              <th class="border-0"></th>
            </tr>
          </thead>
          <tbody>
            {this.state.results.torrents.map(torrent => (
              <tr>
                <td class="align-middle search-name-cell">{torrent.name}</td>
                <td class="align-middle text-right text-muted">{humanFileSize(torrent.size_bytes, true)}</td>
                <td class="align-middle text-right text-success"><i class="fas fa-fw fa-arrow-up d-none d-md-inline mr-1"></i>{torrent.seeders}</td>
                <td class="align-middle text-right text-danger d-none d-md-table-cell"><i class="fas fa-fw fa-arrow-down mr-1"></i>{torrent.leechers}</td>
                <td class="align-middle text-right text-muted d-none d-md-table-cell"
                    data-balloon={`Scraped ${moment(torrent.scraped_date * 1000).fromNow()}`}
                    data-balloon-pos="down">
                      {moment(torrent.created_unix * 1000).fromNow()}
                </td>
                <td class="align-middle text-center">
                  <a class="btn btn-sm no-outline p-1"
                     href={magnetLink(torrent.infohash, torrent.name)}
                     data-balloon="Magnet link"
                     data-balloon-pos="left">
                       <i class="fas fa-fw fa-magnet"></i>
                  </a>
                  <a class="btn btn-sm no-outline p-1 d-none d-md-inline"
                     href={`https://gitlab.com/dessalines/torrents.csv/issues/new?issue[title]=Report%20Torrent%20infohash%20${torrent.infohash}`}
                     target="_blank"
                     data-balloon="Report Torrent"
                     data-balloon-pos="left">
                      <i class="fas fa-fw fa-flag"></i>
                  </a>
                </td>
              </tr>
            ))}
          </tbody>
        </table>
        {this.paginator()}
      </div>
    );
  }

  paginator() {
    return (
      <nav>
        <ul class="pagination justify-content-center">
          <li className={(this.state.searchParams.page == 1) ? "page-item disabled" : "page-item"}>
            <button class="page-link"
              onClick={linkEvent({ i: this, nextPage: false }, this.switchPage)}>
                Previous
            </button>
          </li>
          <li class="page-item">
            <button class="page-link"
              onClick={linkEvent({ i: this, nextPage: true }, this.switchPage)}>
                Next
						</button>
          </li>
        </ul>
      </nav>
    );
  }

  switchPage(a: { i: Search, nextPage: boolean }, event) {
    let newSearch = a.i.state.searchParams;
    newSearch.page += (a.nextPage) ? 1 : -1;
    a.i.props.history.push(`/search/${newSearch.q}/${newSearch.page}`);
  }
}
